from django.conf import settings

LOG_INSPECTOR_FILES = getattr(settings, "LOG_INSPECTOR_FILES", [])
LOG_INSPECTOR_FILES_PATTERN = getattr(settings, "LOG_INSPECTOR_FILES_PATTERN", "*.log*")
LOG_INSPECTOR_FILES_DIR = getattr(settings, "LOG_INSPECTOR_FILES_DIR", "logs/")
LOG_INSPECTOR_PAGE_LENGTH = getattr(settings, "LOG_INSPECTOR_PAGE_LENGTH", 25)
LOG_INSPECTOR_MAX_READ_LINES = getattr(settings, "LOG_INSPECTOR_MAX_READ_LINES", 1000)
LOG_INSPECTOR_FILE_LIST_TITLE = getattr(settings, "LOG_INSPECTOR_FILE_LIST_TITLE", None)
LOG_INSPECTOR_FILE_LIST_STYLES = getattr(settings, "LOG_INSPECTOR_FILE_LIST_STYLES", None)
LOG_INSPECTOR_PATTERNS = getattr(
    settings,
    "LOG_INSPECTOR_PATTERNS",
    ["[INFO]", "[DEBUG]", "[WARNING]", "[ERROR]", "[CRITICAL]"],
)
LOG_INSPECTOR_EXCLUDE_TEXT_PATTERN = getattr(
    settings, "LOG_INSPECTOR_EXCLUDE_TEXT_PATTERN", None
)



import os
import re
from fnmatch import fnmatch
from os.path import isfile

from . import settings


def reverse_readlines(file, buf_size=8192, exclude=None):
    patterns = settings.LOG_INSPECTOR_PATTERNS

    segment = None
    offset = 0
    file.seek(0, os.SEEK_END)
    file_size = remaining_size = file.tell()

    log = []
    while remaining_size > 0:
        offset = min(file_size, offset + buf_size)
        file.seek(file_size - offset)

        buffer = file.read(min(remaining_size, buf_size))

        # remove the file's last "\n" if it exists, only for the first buffer
        if remaining_size == file_size and buffer[-1] == '\n':
            buffer = buffer[:-1]

        remaining_size -= buf_size
        lines = buffer.split('\n')

        # append last chunk's segment to this chunk's last line
        if segment is not None:
            lines[-1] += segment

        segment = lines[0]
        lines = lines[1:]

        for line in reversed(lines):
            log.append(line)

            if any([line.startswith(p) for p in patterns]):
                log_text = '\n'.join(log[::-1])

                if exclude and re.search(exclude, log_text):
                    continue

                yield log_text
                log.clear()

    if segment is not None:
        yield segment


def get_log_entries(filename):
    file_log = os.path.join(settings.LOG_INSPECTOR_FILES_DIR, filename)
    with open(file_log, encoding='utf8', errors='ignore') as file:
        yield from reverse_readlines(file, exclude=settings.LOG_INSPECTOR_EXCLUDE_TEXT_PATTERN)


def get_log_file_names(directory, search=''):
    filenames = []
    matched_file_names = []
    search_pattern = re.compile(search, re.IGNORECASE)

    with os.scandir(directory) as entries:
        for entry in entries:
            matched = fnmatch(entry.name, settings.LOG_INSPECTOR_FILES_PATTERN)
            specified = entry.name in settings.LOG_INSPECTOR_FILES

            if isfile(entry) and (matched or specified):
                filenames.append(entry.name)

    for fn in filenames:
        if search and not search_pattern.search(fn):
            continue

        matched_file_names.append(fn)

    return matched_file_names


def filter_log_entries(log_entries, search=''):
    for entry in log_entries:
        if not entry:
            continue

        search_pattern = re.compile(search, re.IGNORECASE)

        if search and not search_pattern.search(entry):
            continue

        yield entry


def is_valid_filename(filename):
    if filename not in get_log_file_names(settings.LOG_INSPECTOR_FILES_DIR):
        return False

    return True



import io
import os
import zipfile
from itertools import islice

from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.http import Http404, HttpResponse
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView as _TemplateView

from . import settings
from .utils import get_log_entries, get_log_file_names, filter_log_entries, is_valid_filename

HTMX_STOP_POLLING = 286


class TemplateView(_TemplateView):
    @method_decorator(login_required)
    @method_decorator(user_passes_test(lambda u: u.is_superuser))
    def dispatch(self, *args, **kwargs):
        return super(TemplateView, self).dispatch(*args, **kwargs)


class HomeView(TemplateView):
    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        context['custom_title'] = settings.LOG_INSPECTOR_FILE_LIST_TITLE
        context['custom_style_file'] = settings.LOG_INSPECTOR_FILE_LIST_STYLES
        return render(request, 'log_inspector/home.html', context=context)


class LogFilesView(TemplateView):
    def get(self, request, *args, **kwargs):
        search = request.GET.get('search', '')

        filenames = get_log_file_names(settings.LOG_INSPECTOR_FILES_DIR, search)
        filenames.sort()

        context = {'filenames': filenames}
        return render(request, 'log_inspector/log_files.html', context=context)


class LogEntriesView(TemplateView):
    def get(self, request, filename, *args, **kwargs):
        if not is_valid_filename(filename):
            raise Http404

        search = request.GET.get('search', '')
        page = request.GET.get('page', 1)
        action = request.META.get('HTTP_X_ACTION')
        is_live_action = action == 'LIVE'

        log_entries = get_log_entries(filename)
        log_entries = filter_log_entries(log_entries, search)

        max_lines = settings.LOG_INSPECTOR_MAX_READ_LINES if not is_live_action else settings.LOG_INSPECTOR_PAGE_LENGTH
        paginator = Paginator(list(islice(log_entries, max_lines)), settings.LOG_INSPECTOR_PAGE_LENGTH)
        start_index = (int(page) - 1) * paginator.per_page

        try:
            log_entries = paginator.page(page)
        except PageNotAnInteger:
            log_entries = paginator.page(1)
        except EmptyPage:
            log_entries = paginator.page(paginator.num_pages)

        context = {'log_entries': log_entries, 'filename': filename, 'start_index': start_index}
        if is_live_action:
            return render(request, 'log_inspector/log_entries_data.html', context)

        return render(request, 'log_inspector/log_entries_table.html', context, status=HTMX_STOP_POLLING)


class StartLiveView(TemplateView):
    def get(self, request, filename, *args, **kwargs):
        context = {'filename': filename}
        return render(request, 'log_inspector/start_live.html', context=context)


class DownloadLogFileView(TemplateView):
    def get(self, request, filename, *args, **kwargs):
        if not is_valid_filename(filename):
            raise Http404

        zip_buffer = io.BytesIO()
        with zipfile.ZipFile(zip_buffer, 'w') as zip_file:
            zip_file.write(os.path.join(settings.LOG_INSPECTOR_FILES_DIR, filename))

        basename, _ = os.path.splitext(filename)
        response = HttpResponse(content_type='application/zip')
        response['Content-Disposition'] = f'attachment; filename="{basename}.zip"'
        response.write(zip_buffer.getvalue())
        return response



import django

if django.VERSION >= (2, 0):
    from django.urls import re_path
else:
    from django.conf.urls import url as re_path

from .views import LogEntriesView, LogFilesView, HomeView, StartLiveView, DownloadLogFileView

app_name = "log_inspector"

urlpatterns = [
    re_path(r"^$", HomeView.as_view(), name='home'),
    re_path(r'^log-files/$', LogFilesView.as_view(), name='log_files'),
    re_path(r'^log-entries/(?P<filename>[\w\.-]+)/$', LogEntriesView.as_view(), name='log_entries'),
    re_path(r'^start-live/(?P<filename>[\w\.-]+)/$', StartLiveView.as_view(), name='start_live'),
    re_path(r'^download/(?P<filename>[\w\.-]+)/$', DownloadLogFileView.as_view(), name='download'),
]
